import re
from datetime import datetime, timedelta

from series.get.model.show import Show

from sqlalchemy.sql.expression import and_

from amino import Maybe, Map, List
from series import canonicalize
from series.get.tvdb import Tvdb

from series.db_facade import DbFacade, exclusive, commit
from series.util import datetime_to_unix


class ShowsFacade(Tvdb, DbFacade):

    @property
    def main_type(self):
        return Show

    @property
    def shows(self):
        return self.ordered

    @exclusive
    def name_exists(self, name):
        return self.filter_by(canonical_name=canonicalize(name)).count() > 0

    def id_param(self, showid):
        return (dict(etvdb_id=showid) if self.use_etvdb else
                dict(rage_id=showid))

    @exclusive
    def add(self, name, show):
        data = dict(
            name=show.name,
            search_name=show.name,
            canonical_name=canonicalize(name),
        )
        data.update(**self.id_param(show.showid))
        if show.latest_episode is not None:
            data.update(latest_episode=show.latest_episode.number,
                        latest_season=show.latest_episode.season)
        self._db.add(Show(**data))

    @commit
    def update(self, id_, data):
        query = self.filter_by(id=id_)
        if query.count() > 0:
            show = query.first()
            for key, value in data.items():
                setattr(show, key, value)
            return show
        else:
            self.log.debug(
                'Tried to update nonexistent show with id {}'.format(id_)
            )

    def update_by_id(self, id, **data):
        return self.update(id, data)

    @commit
    def delete_by_id(self, id):
        query = self.filter_by(id=id)
        if query.count() > 0:
            show = query.first()
            self._db.delete(show)
        else:
            self.log.error(
                'Tried to delete nonexistent show with id {}'.format(id)
            )

    @commit
    def delete_by_sid(self, showid):
        query = self.filter_by(**self.id_param(showid))
        if query.count() > 0:
            show = query.first()
            self._db.delete(show)
        else:
            self.log.error(
                'Tried to delete nonexistent show with sid {}'.format(showid)
            )

    @commit
    def delete(self, showid):
        query = self.filter_by(id=showid)
        if query.count() > 0:
            show = query.first()
            self._db.delete(show)
        else:
            self.log.debug(
                'Tried to delete nonexistent show with id {}'.format(showid)
            )

    @property  # type: ignore
    @exclusive
    def all(self):
        return List.wrap(self.shows)

    def filter_by_regex(self, regex):
        r = re.compile(regex)
        return filter(lambda s: r.search(s.canonical_name), self.all)

    @exclusive
    def find_by_name(self, name):
        return Maybe(self.filter_by(canonical_name=name).first())

    @exclusive
    def find_by_metadata(self, **data):
        return Maybe(self.filter_by(**data).first())

    @property
    def status(self):
        now = datetime.now().replace(hour=0, minute=0, second=0)
        thresh = now + timedelta(days=7)
        filt = and_(Show.next_episode_stamp >= datetime_to_unix(now),
                    Show.next_episode_stamp < datetime_to_unix(thresh))
        nxt = List.wrap(self.filter(filt))
        def info(s):
            return Map(series=s.canonical_name,
                       episode=s.next_episode,
                       season=s.season,
                       airdate=s.next_episode_day)
        return Map(next=nxt / info)

    @property  # type: ignore
    @exclusive
    def count(self):
        return self.shows.count()

__all__ = ['ShowsFacade']



"""add metadata attributes

Revision ID: 26134e26286
Revises: 213b486db87
Create Date: 2014-06-27 05:21:34.613857

"""

# revision identifiers, used by Alembic.
revision = '26134e26286'
down_revision = '213b486db87'

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('episode', sa.Column('metadata_failures', sa.Integer(),
                                       nullable=True, server_default='0'))
    op.add_column('episode', sa.Column('metadata_fetched', sa.Boolean(),
                                       nullable=True, server_default='0'))
    op.add_column('episode', sa.Column('overview', sa.String(), nullable=True,
                                       server_default=''))
    op.add_column('season', sa.Column('metadata_failures', sa.Integer(),
                                      nullable=True, server_default='0'))
    op.add_column('season', sa.Column('metadata_fetched', sa.Boolean(),
                                      nullable=True, server_default='0'))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('season', 'metadata_fetched')
    op.drop_column('season', 'metadata_failures')
    op.drop_column('episode', 'overview')
    op.drop_column('episode', 'metadata_fetched')
    op.drop_column('episode', 'metadata_failures')
    ### end Alembic commands ###



import sys

from golgi import cli

from series.client.cli import HTTPCLI
from series.library.client.rest_api import LibClient
from series.library import SeriesLibraryD


class SeriesLibraryC(HTTPCLI):

    @property
    def _client(self):
        return LibClient()


@cli(positional=(('cli_cmd', 1), ('cli_params', '*')))
def libc():
    if not SeriesLibraryC().run():
        sys.exit(1)


@cli()
def libd():
    SeriesLibraryD().run()

__all__ = ['libc', 'libd']



from golgi.config import ListConfigOption, DictConfigOption, ConfigOption

metadata = dict(parents=['series'])


def reset_config():
    return {
        'subsync': dict(
            splitchar='|', base_url=None, cookies=DictConfigOption(),
            series_url_map=DictConfigOption(), only_latest=False,
            episodes=ListConfigOption(positional=True),
            cli_dir=ConfigOption(positional=True),
        )
    }



"""add ReleaseMonitor._resolutions

Revision ID: a49797d68d62
Revises: 566cc0cdd00a
Create Date: 2016-05-30 20:33:37.764631

"""

# revision identifiers, used by Alembic.
revision = 'a49797d68d62'
down_revision = '566cc0cdd00a'

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('release_monitor', sa.Column('_resolutions', sa.String(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('release_monitor', sa.Column('torrent_cached', sa.BOOLEAN(), nullable=True))
    ### end Alembic commands ###
